@@ -11,6 +11,7 @@
 #include <librsvg/rsvg-cairo.h>
 #endif
 
+#include "eog-config-keys.h"
 #include "eog-marshal.h"
 #include "eog-scroll-view.h"
 #include "eog-debug.h"
@@ -72,6 +73,12 @@ static GtkTargetEntry target_table[] = {
 	{ "text/uri-list", 0, 0},
 };
 
+enum {
+	PROP_0,
+	PROP_USE_BG_COLOR,
+	PROP_BACKGROUND_COLOR
+};
+
 /* Private part of the EogScrollView structure */
 struct _EogScrollViewPrivate {
 	/* some widgets we rely on */
@@ -138,6 +145,10 @@ struct _EogScrollViewPrivate {
 	/* the type of the cursor we are currently showing */
 	EogScrollViewCursor cursor;
 
+	gboolean  use_bg_color;
+	GdkColor *background_color;
+	GdkColor *override_bg_color;
+
 	cairo_surface_t *background_surface;
 };
 
@@ -258,35 +269,40 @@ update_scrollbar_values (EogScrollView *view)
 	EogScrollViewPrivate *priv;
 	int scaled_width, scaled_height;
 	int xofs, yofs;
-	GtkAllocation *allocation;
+	gdouble page_size,page_increment,step_increment;
+	gdouble lower, upper, value;
+	GtkAllocation allocation;
 
 	priv = view->priv;
 
-	if (!gtk_widget_get_visible (GTK_WIDGET (priv->hbar)) && !gtk_widget_get_visible (GTK_WIDGET (priv->vbar)))
+	if (!gtk_widget_get_visible (GTK_WIDGET (priv->hbar))
+	    && !gtk_widget_get_visible (GTK_WIDGET (priv->vbar)))
 		return;
 
 	compute_scaled_size (view, priv->zoom, &scaled_width, &scaled_height);
-	allocation = &GTK_WIDGET (priv->display)->allocation;
+	gtk_widget_get_allocation (GTK_WIDGET (priv->display), &allocation);
 
 	if (gtk_widget_get_visible (GTK_WIDGET (priv->hbar))) {
 		/* Set scroll increments */
-		priv->hadj->page_size = MIN (scaled_width, allocation->width);
-		priv->hadj->page_increment = allocation->width / 2;
-		priv->hadj->step_increment = SCROLL_STEP_SIZE;
+		page_size = MIN (scaled_width, allocation.width);
+		page_increment = allocation.width / 2;
+		step_increment = SCROLL_STEP_SIZE;
 
 		/* Set scroll bounds and new offsets */
-		priv->hadj->lower = 0;
-		priv->hadj->upper = scaled_width;
-		xofs = CLAMP (priv->xofs, 0, priv->hadj->upper - priv->hadj->page_size);
-		if (priv->hadj->value != xofs) {
-			priv->hadj->value = xofs;
+		lower = 0;
+		upper = scaled_width;
+		xofs = CLAMP (priv->xofs, 0, upper - page_size);
+		if (gtk_adjustment_get_value (priv->hadj) != xofs) {
+			value = xofs;
 			priv->xofs = xofs;
 
 			g_signal_handlers_block_matched (
 				priv->hadj, G_SIGNAL_MATCH_DATA,
 				0, 0, NULL, NULL, view);
 
-			gtk_adjustment_changed (priv->hadj);
+			gtk_adjustment_configure (priv->hadj, value, lower,
+						  upper, step_increment,
+						  page_increment, page_size);
 
 			g_signal_handlers_unblock_matched (
 				priv->hadj, G_SIGNAL_MATCH_DATA,
@@ -295,23 +311,25 @@ update_scrollbar_values (EogScrollView *view)
 	}
 
 	if (gtk_widget_get_visible (GTK_WIDGET (priv->vbar))) {
-		priv->vadj->page_size = MIN (scaled_height, allocation->height);
-		priv->vadj->page_increment = allocation->height / 2;
-		priv->vadj->step_increment = SCROLL_STEP_SIZE;
+		page_size = MIN (scaled_height, allocation.height);
+		page_increment = allocation.height / 2;
+		step_increment = SCROLL_STEP_SIZE;
 
-		priv->vadj->lower = 0;
-		priv->vadj->upper = scaled_height;
-		yofs = CLAMP (priv->yofs, 0, priv->vadj->upper - priv->vadj->page_size);
+		lower = 0;
+		upper = scaled_height;
+		yofs = CLAMP (priv->yofs, 0, upper - page_size);
 
-		if (priv->vadj->value != yofs) {
-			priv->vadj->value = yofs;
+		if (gtk_adjustment_get_value (priv->vadj) != yofs) {
+			value = yofs;
 			priv->yofs = yofs;
 
 			g_signal_handlers_block_matched (
 				priv->vadj, G_SIGNAL_MATCH_DATA,
 				0, 0, NULL, NULL, view);
 
-			gtk_adjustment_changed (priv->vadj);
+			gtk_adjustment_configure (priv->vadj, value, lower,
+						  upper, step_increment,
+						  page_increment, page_size);
 
 			g_signal_handlers_unblock_matched (
 				priv->vadj, G_SIGNAL_MATCH_DATA,
@@ -387,8 +405,11 @@ check_scrollbar_visibility (EogScrollView *view, GtkAllocation *alloc)
 		width = alloc->width;
 		height = alloc->height;
 	} else {
-		width = GTK_WIDGET (view)->allocation.width;
-		height = GTK_WIDGET (view)->allocation.height;
+		GtkAllocation allocation;
+
+		gtk_widget_get_allocation (GTK_WIDGET (view), &allocation);
+		width = allocation.width;
+		height = allocation.height;
 	}
 
 	compute_scaled_size (view, priv->zoom, &img_width, &img_height);
@@ -732,13 +753,13 @@ static void
 paint_rectangle (EogScrollView *view, EogIRect *rect, GdkInterpType interp_type)
 {
 	EogScrollViewPrivate *priv;
+	GdkPixbuf *tmp;
+	char *str;
+	GtkAllocation allocation;
 	int scaled_width, scaled_height;
-	int width, height;
 	int xofs, yofs;
 	EogIRect r, d;
-	GdkPixbuf *tmp;
 	int check_size;
-	char *str;
 	guint32 check_1 = 0;
 	guint32 check_2 = 0;
 
@@ -749,28 +770,27 @@ paint_rectangle (EogScrollView *view, EogIRect *rect, GdkInterpType interp_type)
 
 	compute_scaled_size (view, priv->zoom, &scaled_width, &scaled_height);
 
-	width = GTK_WIDGET (priv->display)->allocation.width;
-	height = GTK_WIDGET (priv->display)->allocation.height;
+	gtk_widget_get_allocation (GTK_WIDGET (priv->display), &allocation);
 
 	if (scaled_width < 1 || scaled_height < 1)
 	{
 		r.x0 = 0;
 		r.y0 = 0;
-		r.x1 = width;
-		r.y1 = height;
+		r.x1 = allocation.width;
+		r.y1 = allocation.height;
 		paint_background (view, &r, rect);
 		return;
 	}
 
 	/* Compute image offsets with respect to the window */
 
-	if (scaled_width <= width)
-		xofs = (width - scaled_width) / 2;
+	if (scaled_width <= allocation.width)
+		xofs = (allocation.width - scaled_width) / 2;
 	else
 		xofs = -priv->xofs;
 
-	if (scaled_height <= height)
-		yofs = (height - scaled_height) / 2;
+	if (scaled_height <= allocation.height)
+		yofs = (allocation.height - scaled_height) / 2;
 	else
 		yofs = -priv->yofs;
 
@@ -783,7 +803,7 @@ paint_rectangle (EogScrollView *view, EogIRect *rect, GdkInterpType interp_type)
 	if (yofs > 0) {
 		r.x0 = 0;
 		r.y0 = 0;
-		r.x1 = width;
+		r.x1 = allocation.width;
 		r.y1 = yofs;
 		paint_background (view, &r, rect);
 	}
@@ -801,7 +821,7 @@ paint_rectangle (EogScrollView *view, EogIRect *rect, GdkInterpType interp_type)
 	if (xofs >= 0) {
 		r.x0 = xofs + scaled_width;
 		r.y0 = yofs;
-		r.x1 = width;
+		r.x1 = allocation.width;
 		r.y1 = yofs + scaled_height;
 		if (r.x0 < r.x1)
 			paint_background (view, &r, rect);
@@ -811,8 +831,8 @@ paint_rectangle (EogScrollView *view, EogIRect *rect, GdkInterpType interp_type)
 	if (yofs >= 0) {
 		r.x0 = 0;
 		r.y0 = yofs + scaled_height;
-		r.x1 = width;
-		r.y1 = height;
+		r.x1 = allocation.width;
+		r.y1 = allocation.height;
 		if (r.y0 < r.y1)
 			paint_background (view, &r, rect);
 	}
@@ -964,6 +984,7 @@ request_paint_area (EogScrollView *view, GdkRectangle *area)
 {
 	EogScrollViewPrivate *priv;
 	EogIRect r;
+	GtkAllocation allocation;
 
 	priv = view->priv;
 
@@ -973,10 +994,11 @@ request_paint_area (EogScrollView *view, GdkRectangle *area)
 	if (!gtk_widget_is_drawable (priv->display))
 		return;
 
+	gtk_widget_get_allocation (GTK_WIDGET (priv->display), &allocation);
 	r.x0 = MAX (0, area->x);
 	r.y0 = MAX (0, area->y);
-	r.x1 = MIN (GTK_WIDGET (priv->display)->allocation.width, area->x + area->width);
-	r.y1 = MIN (GTK_WIDGET (priv->display)->allocation.height, area->y + area->height);
+	r.x1 = MIN (allocation.width, area->x + area->width);
+	r.y1 = MIN (allocation.height, area->y + area->height);
 
 	eog_debug_message (DEBUG_WINDOW, "r: %i, %i, %i, %i\n", r.x0, r.y0, r.x1, r.y1);
 
@@ -1026,9 +1048,9 @@ static void
 scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 {
 	EogScrollViewPrivate *priv;
+	GtkAllocation allocation;
 	int xofs, yofs;
 	GdkWindow *window;
-	int width, height;
 	int src_x, src_y;
 	int dest_x, dest_y;
 	int twidth, theight;
@@ -1037,13 +1059,15 @@ scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 
 	/* Check bounds & Compute offsets */
 	if (gtk_widget_get_visible (priv->hbar)) {
-		x = CLAMP (x, 0, priv->hadj->upper - priv->hadj->page_size);
+		x = CLAMP (x, 0, gtk_adjustment_get_upper (priv->hadj)
+				 - gtk_adjustment_get_page_size (priv->hadj));
 		xofs = x - priv->xofs;
 	} else
 		xofs = 0;
 
 	if (gtk_widget_get_visible (priv->vbar)) {
-		y = CLAMP (y, 0, priv->vadj->upper - priv->vadj->page_size);
+		y = CLAMP (y, 0, gtk_adjustment_get_upper (priv->vadj)
+				 - gtk_adjustment_get_page_size (priv->vadj));
 		yofs = y - priv->yofs;
 	} else
 		yofs = 0;
@@ -1057,10 +1081,9 @@ scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 	if (!gtk_widget_is_drawable (priv->display))
 		goto out;
 
-	width = GTK_WIDGET (priv->display)->allocation.width;
-	height = GTK_WIDGET (priv->display)->allocation.height;
+	gtk_widget_get_allocation (GTK_WIDGET (priv->display), &allocation);
 
-	if (abs (xofs) >= width || abs (yofs) >= height) {
+	if (abs (xofs) >= allocation.width || abs (yofs) >= allocation.height) {
 		gtk_widget_queue_draw (GTK_WIDGET (priv->display));
 		goto out;
 	}
@@ -1069,8 +1092,8 @@ scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 
 	/* Ensure that the uta has the full size */
 
-	twidth = (width + EOG_UTILE_SIZE - 1) >> EOG_UTILE_SHIFT;
-	theight = (height + EOG_UTILE_SIZE - 1) >> EOG_UTILE_SHIFT;
+	twidth = (allocation.width + EOG_UTILE_SIZE - 1) >> EOG_UTILE_SHIFT;
+	theight = (allocation.height + EOG_UTILE_SIZE - 1) >> EOG_UTILE_SHIFT;
 
 	if (priv->uta)
 		g_assert (priv->idle_id != 0);
@@ -1090,7 +1113,8 @@ scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 	uta_copy_area (priv->uta,
 		       src_x, src_y,
 		       dest_x, dest_y,
-		       width - abs (xofs), height - abs (yofs));
+		       allocation.width - abs (xofs),
+		       allocation.height - abs (yofs));
 
 	/* Scroll the window area and process exposure synchronously. */
 
@@ -1108,11 +1132,8 @@ scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 		priv->vadj, G_SIGNAL_MATCH_DATA,
 		0, 0, NULL, NULL, view);
 
-	priv->hadj->value = x;
-	priv->vadj->value = y;
-
-	gtk_adjustment_value_changed (priv->hadj);
-	gtk_adjustment_value_changed (priv->vadj);
+	gtk_adjustment_set_value (priv->hadj, x);
+	gtk_adjustment_set_value (priv->vadj, y);
 
 	g_signal_handlers_unblock_matched (
 		priv->hadj, G_SIGNAL_MATCH_DATA,
@@ -1146,7 +1167,8 @@ adjustment_changed_cb (GtkAdjustment *adj, gpointer data)
 	view = EOG_SCROLL_VIEW (data);
 	priv = view->priv;
 
-	scroll_to (view, priv->hadj->value, priv->vadj->value, FALSE);
+	scroll_to (view, gtk_adjustment_get_value (priv->hadj),
+		   gtk_adjustment_get_value (priv->vadj), FALSE);
 }
 
 
@@ -1199,8 +1221,8 @@ set_zoom (EogScrollView *view, double zoom,
 	  gboolean have_anchor, int anchorx, int anchory)
 {
 	EogScrollViewPrivate *priv;
+	GtkAllocation allocation;
 	int xofs, yofs;
-	int disp_width, disp_height;
 	double x_rel, y_rel;
 
 	g_assert (zoom > 0.0);
@@ -1222,20 +1244,19 @@ set_zoom (EogScrollView *view, double zoom,
 
 	priv->zoom_mode = ZOOM_MODE_FREE;
 
-	disp_width = GTK_WIDGET (priv->display)->allocation.width;
-	disp_height = GTK_WIDGET (priv->display)->allocation.height;
+	gtk_widget_get_allocation (GTK_WIDGET (priv->display), &allocation);
 
 	/* compute new xofs/yofs values */
 	if (have_anchor) {
-		x_rel = (double) anchorx / disp_width;
-		y_rel = (double) anchory / disp_height;
+		x_rel = (double) anchorx / allocation.width;
+		y_rel = (double) anchory / allocation.height;
 	} else {
 		x_rel = 0.5;
 		y_rel = 0.5;
 	}
 
 	compute_center_zoom_offsets (view, priv->zoom, zoom,
-				     disp_width, disp_height,
+				     allocation.width, allocation.height,
 				     x_rel, y_rel,
 				     &xofs, &yofs);
 
@@ -1265,23 +1286,22 @@ static void
 set_zoom_fit (EogScrollView *view)
 {
 	EogScrollViewPrivate *priv;
+	GtkAllocation allocation;
 	double new_zoom;
-	int width, height;
 
 	priv = view->priv;
 
 	priv->zoom_mode = ZOOM_MODE_FIT;
 
-	if (!GTK_WIDGET_MAPPED (GTK_WIDGET (view)))
+	if (!gtk_widget_get_mapped (GTK_WIDGET (view)))
 		return;
 
 	if (priv->pixbuf == NULL)
 		return;
 
-	width = GTK_WIDGET (priv->display)->allocation.width;
-	height = GTK_WIDGET (priv->display)->allocation.height;
+	gtk_widget_get_allocation (GTK_WIDGET(priv->display), &allocation);
 
-	new_zoom = zoom_fit_scale (width, height,
+	new_zoom = zoom_fit_scale (allocation.width, allocation.height,
 				   gdk_pixbuf_get_width (priv->pixbuf),
 				   gdk_pixbuf_get_height (priv->pixbuf),
 				   priv->upscale);
@@ -1310,11 +1330,11 @@ display_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
 	EogScrollView *view;
 	EogScrollViewPrivate *priv;
+	GtkAllocation allocation;
 	gboolean do_zoom;
 	double zoom;
 	gboolean do_scroll;
 	int xofs, yofs;
-	int width, height;
 
 	view = EOG_SCROLL_VIEW (data);
 	priv = view->priv;
@@ -1324,8 +1344,7 @@ display_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 	xofs = yofs = 0;
 	zoom = 1.0;
 
-	width = GTK_WIDGET (priv->display)->allocation.width;
-	height = GTK_WIDGET (priv->display)->allocation.height;
+	gtk_widget_get_allocation (GTK_WIDGET (priv->display), &allocation);
 
 	/* EogScrollView doesn't handle/have any Alt+Key combos */
 	if (event->state & GDK_MOD1_MASK) {
@@ -1342,11 +1361,11 @@ display_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 	case GDK_Page_Up:
 		do_scroll = TRUE;
 		if (event->state & GDK_CONTROL_MASK) {
-			xofs = -(width * 3) / 4;
+			xofs = -(allocation.width * 3) / 4;
 			yofs = 0;
 		} else {
 			xofs = 0;
-			yofs = -(height * 3) / 4;
+			yofs = -(allocation.height * 3) / 4;
 		}
 		break;
 
@@ -1359,11 +1378,11 @@ display_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 	case GDK_Page_Down:
 		do_scroll = TRUE;
 		if (event->state & GDK_CONTROL_MASK) {
-			xofs = (width * 3) / 4;
+			xofs = (allocation.width * 3) / 4;
 			yofs = 0;
 		} else {
 			xofs = 0;
-			yofs = (height * 3) / 4;
+			yofs = (allocation.height * 3) / 4;
 		}
 		break;
 
@@ -1517,9 +1536,9 @@ eog_scroll_view_scroll_event (GtkWidget *widget, GdkEventScroll *event, gpointer
 	priv = view->priv;
 
 	/* Compute zoom factor and scrolling offsets; we'll only use either of them */
-
-	xofs = priv->hadj->page_increment / 2; /* same as in gtkscrolledwindow.c */
-	yofs = priv->vadj->page_increment / 2;
+	/* same as in gtkscrolledwindow.c */
+	xofs = gtk_adjustment_get_page_increment (priv->hadj) / 2;
+	yofs = gtk_adjustment_get_page_increment (priv->vadj) / 2;
 
 	switch (event->direction) {
 	case GDK_SCROLL_UP:
@@ -1942,6 +1961,11 @@ eog_scroll_view_set_transparency (EogScrollView *view, EogTransparencyStyle styl
 	}
 
 	if (changed && priv->pixbuf != NULL && gdk_pixbuf_get_has_alpha (priv->pixbuf)) {
+		if (priv->background_surface) {
+			cairo_surface_destroy (priv->background_surface);
+			/* Will be recreated if needed during redraw */
+			priv->background_surface = NULL;
+		}
 		gtk_widget_queue_draw (GTK_WIDGET (priv->display));
 	}
 }
@@ -2160,13 +2184,55 @@ eog_scroll_view_scrollbars_visible (EogScrollView *view)
     object creation/freeing
   ---------------------------------*/
 
+static gboolean
+sv_string_to_color_mapping (GValue   *value,
+			    GVariant *variant,
+			    gpointer  user_data)
+{
+	GdkColor color;
+
+	g_return_val_if_fail (g_variant_is_of_type (variant, G_VARIANT_TYPE_STRING), FALSE);
+
+	if (gdk_color_parse (g_variant_get_string (variant, NULL), &color)) {
+		g_value_set_boxed (value, &color);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static GVariant*
+sv_color_to_string_mapping (const GValue       *value,
+			    const GVariantType *expected_type,
+			    gpointer            user_data)
+{
+	GVariant *variant = NULL;
+	GdkColor *color;
+	gchar *hex_val;
+
+	g_return_val_if_fail (G_VALUE_TYPE (value) == GDK_TYPE_COLOR, NULL);
+	g_return_val_if_fail (g_variant_type_equal (expected_type, G_VARIANT_TYPE_STRING), NULL);
+
+	color = g_value_get_boxed (value);
+	hex_val = g_strdup_printf ("#%02X%02X%02X",
+				   color->red / 256,
+				   color->green / 256,
+				   color->blue / 256);
+	variant = g_variant_new_string (hex_val);
+	g_free (hex_val);
+
+	return variant;
+}
+
 
 static void
 eog_scroll_view_init (EogScrollView *view)
 {
+	GSettings *settings;
 	EogScrollViewPrivate *priv;
 
 	priv = view->priv = EOG_SCROLL_VIEW_GET_PRIVATE (view);
+	settings = g_settings_new (EOG_CONF_VIEW);
 
 	priv->zoom = 1.0;
 	priv->min_zoom = MIN_ZOOM_FACTOR;
@@ -2184,6 +2250,17 @@ eog_scroll_view_init (EogScrollView *view)
 	priv->transp_color = 0;
 	priv->cursor = EOG_SCROLL_VIEW_CURSOR_NORMAL;
 	priv->menu = NULL;
+	
+	g_settings_bind (settings, EOG_CONF_VIEW_USE_BG_COLOR, view,
+			 "use-background-color", G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind_with_mapping (settings, EOG_CONF_VIEW_BACKGROUND_COLOR,
+				      view, "background-color",
+				      G_SETTINGS_BIND_DEFAULT,
+				      sv_string_to_color_mapping,
+				      sv_color_to_string_mapping, NULL, NULL);
+	g_object_unref (settings);
+
+	priv->override_bg_color = NULL;
 	priv->background_surface = NULL;
 }
 
@@ -2208,6 +2285,16 @@ eog_scroll_view_dispose (GObject *object)
 		priv->idle_id = 0;
 	}
 
+	if (priv->background_color != NULL) {
+		gdk_color_free (priv->background_color);
+		priv->background_color = NULL;
+	}
+
+	if (priv->override_bg_color != NULL) {
+		gdk_color_free (priv->override_bg_color);
+		priv->override_bg_color = NULL;
+	}
+
 	if (priv->background_surface != NULL) {
 		cairo_surface_destroy (priv->background_surface);
 		priv->background_surface = NULL;
@@ -2218,6 +2305,59 @@ eog_scroll_view_dispose (GObject *object)
 	G_OBJECT_CLASS (eog_scroll_view_parent_class)->dispose (object);
 }
 
+static void
+eog_scroll_view_get_property (GObject *object, guint property_id,
+			      GValue *value, GParamSpec *pspec)
+{
+	EogScrollView *view;
+	EogScrollViewPrivate *priv;
+
+	g_return_if_fail (EOG_IS_SCROLL_VIEW (object));
+
+	view = EOG_SCROLL_VIEW (object);
+	priv = view->priv;
+
+	switch (property_id) {
+	case PROP_USE_BG_COLOR:
+		g_value_set_boolean (value, priv->use_bg_color);
+		break;
+	case PROP_BACKGROUND_COLOR:
+		//FIXME: This doesn't really handle the NULL color.
+		g_value_set_boxed (value, priv->background_color);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+	}
+}
+
+static void
+eog_scroll_view_set_property (GObject *object, guint property_id,
+			      const GValue *value, GParamSpec *pspec)
+{
+	EogScrollView *view;
+	EogScrollViewPrivate *priv;
+
+	g_return_if_fail (EOG_IS_SCROLL_VIEW (object));
+
+	view = EOG_SCROLL_VIEW (object);
+	priv = view->priv;
+
+	switch (property_id) {
+	case PROP_USE_BG_COLOR:
+		eog_scroll_view_set_use_bg_color (view, g_value_get_boolean (value));
+		break;
+	case PROP_BACKGROUND_COLOR:
+	{
+		const GdkColor *color = g_value_get_boxed (value);
+		eog_scroll_view_set_background_color (view, color);
+		break;
+	}
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+	}
+}
+
+
 static void
 eog_scroll_view_class_init (EogScrollViewClass *klass)
 {
@@ -2228,6 +2368,26 @@ eog_scroll_view_class_init (EogScrollViewClass *klass)
 	widget_class = (GtkWidgetClass*) klass;
 
 	gobject_class->dispose = eog_scroll_view_dispose;
+        gobject_class->set_property = eog_scroll_view_set_property;
+        gobject_class->get_property = eog_scroll_view_get_property;
+
+	/**
+	 * EogScrollView:background-color:
+	 *
+	 * This is the default background color used for painting the background
+	 * of the image view. If set to %NULL the color is determined by the
+	 * active GTK theme.
+	 */
+	g_object_class_install_property (
+		gobject_class, PROP_BACKGROUND_COLOR,
+		g_param_spec_boxed ("background-color", NULL, NULL,
+				    GDK_TYPE_COLOR,
+				    G_PARAM_READWRITE | G_PARAM_STATIC_NAME));
+
+	g_object_class_install_property (
+		gobject_class, PROP_USE_BG_COLOR,
+		g_param_spec_boolean ("use-background-color", NULL, NULL, FALSE,
+				      G_PARAM_READWRITE | G_PARAM_STATIC_NAME));
 
 	view_signals [SIGNAL_ZOOM_CHANGED] =
 		g_signal_new ("zoom_changed",
@@ -2430,13 +2590,93 @@ eog_scroll_view_set_popup (EogScrollView *view,
 			  G_CALLBACK (view_on_button_press_event_cb), NULL);
 }
 
+static gboolean
+_eog_gdk_color_equal0 (const GdkColor *a, const GdkColor *b)
+{
+	if (a == NULL || b == NULL)
+		return (a == b);
+
+	return gdk_color_equal (a, b);
+}
+
+static gboolean
+_eog_replace_gdk_color (GdkColor **dest, const GdkColor *new)
+{
+	GdkColor *old = *dest;
+
+	if (_eog_gdk_color_equal0 (old, new))
+		return FALSE;
+
+	if (old != NULL)
+		gdk_color_free (old);
+
+	*dest = (new) ? gdk_color_copy (new) : NULL;
+
+	return TRUE;
+}
+
+static void
+_eog_scroll_view_update_bg_color (EogScrollView *view)
+{
+	const GdkColor *selected;
+	EogScrollViewPrivate *priv = view->priv;
+
+	if (!priv->use_bg_color)
+		selected = NULL;
+	else if (priv->override_bg_color)
+		selected = priv->override_bg_color;
+	else
+		selected = priv->background_color;
+
+	if (priv->transp_style == EOG_TRANSP_BACKGROUND
+	    && priv->background_surface != NULL) {
+		/* Delete the SVG background to have it recreated with
+		 * the correct color during the next SVG redraw */
+		cairo_surface_destroy (priv->background_surface);
+		priv->background_surface = NULL;
+	}
+
+	gtk_widget_modify_bg (GTK_WIDGET (view),
+			      GTK_STATE_NORMAL,
+			      selected);
+}
+
 void
-eog_scroll_view_set_bg_color (EogScrollView *view,
-			      GdkColor *color)
+eog_scroll_view_set_background_color (EogScrollView *view,
+				      const GdkColor *color)
 {
 	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
 
-	gtk_widget_modify_bg (GTK_WIDGET (view), GTK_STATE_NORMAL, color);
+	if (_eog_replace_gdk_color (&view->priv->background_color, color))
+		_eog_scroll_view_update_bg_color (view);
+}
+
+void
+eog_scroll_view_override_bg_color (EogScrollView *view,
+				   const GdkColor *color)
+{
+	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
+
+	if (_eog_replace_gdk_color (&view->priv->override_bg_color, color))
+		_eog_scroll_view_update_bg_color (view);
+}
+
+void
+eog_scroll_view_set_use_bg_color (EogScrollView *view, gboolean use)
+{
+	EogScrollViewPrivate *priv;
+
+	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
+
+	priv = view->priv;
+
+	if (use != priv->use_bg_color) {
+		priv->use_bg_color = use;
+
+		_eog_scroll_view_update_bg_color (view);
+
+		g_object_notify (G_OBJECT (view), "use-background-color");
+	}
 }
 
 void
